import _ from "lodash";
import {removeStartingAndEndingSlash} from "./utils/encoders";
import {getWizardSelectedParams} from "./utils/sync-utils"
import {SERVER_TYPES} from './consts';

function getPropFromState(state, key, defaultValue) {
    return state.syncReducer[key] ? state.syncReducer[key] : defaultValue;
}

function fillWizardData(mappedProps, ownProps) {
    if (!mappedProps._wizard || !ownProps.match || !ownProps.match.params) return mappedProps;

    const {stepsMapping, currentStep} = mappedProps._wizard;
    //remove beginning slash and ending slash
    const currentPath = removeStartingAndEndingSlash(ownProps.location.pathname);

    const currentPathSegments = currentPath.split("/");

    const currentPathStep = stepsMapping[currentStep];
    const currentPathIndex = currentPathSegments.indexOf(currentPathStep);
    const newPath = currentPathIndex > -1 ? currentPathSegments.slice(0, currentPathIndex + 1).join("/") : currentPath;

    const nextStep = (suffix, history) => {
        history.push(`/${newPath}${suffix}`);
    };
    const {_wizard: {urlParams}} = mappedProps;
    //build the wizard params out of the current url params, or if not exist, with params kept in wizard.urlParams - to allow filling the wizard when moving forward with all that was already selected
    const wizardParams = Object.assign({}, urlParams, ownProps.match.params);
    // console.log(`@@@@@@@@@@@@@@  mapStateToProps ownProps.match.params:${JSON.stringify(ownProps.match.params)} - urlParams:${JSON.stringify(urlParams)}`);
    const selectedWizardParams = getWizardSelectedParams(mappedProps, wizardParams);
    // console.log(`%%%%%%%%%%% selectedWizardParams:${JSON.stringify(selectedWizardParams)}`);

    mappedProps._wizard = mappedProps._wizard.set("nextStep", nextStep).updateIn(["selectedWizardParams"], currentParams => {
        return Object.assign({}, currentParams, selectedWizardParams);
    });

    // console.log(`&&&&&&&&&&&&&& selectedWizardParams from state: ${JSON.stringify(mappedProps._wizard.selectedWizardParams)}`);
    return mappedProps;
}

function buildErrorsList(state) {
    const errorsMap = state.syncReducer._generalErrors._errorsMap;
    return {
        _errors: _.orderBy(_.values(errorsMap), "time", "desc"),
        _errorsPanelOpen: state.syncReducer._generalErrors._errorsPanelOpen,
        _errorsPanelSize: state.syncReducer._generalErrors._errorsPanelSize
    }
}
function buildServersList(state) {
    return Object.values(SERVER_TYPES).reduce((res, k) => {
            res[k] = (state.syncReducer._servers && state.syncReducer._servers[k]) ? state.syncReducer._servers[k] : {inProgress: true};
            return res;
        }, {});
}

export default function (state, ownProps) {


    return fillWizardData({
        auth: getPropFromState(state, 'auth', {succeeded: false}),
        _defineDataBroker: getPropFromState(state, "_defineDataBroker", {}),
        _servers: buildServersList(state),
        _gcpProjects: getPropFromState(state, "_gcpProjects", {inProgress: true}),
        _azureStorageAccounts: getPropFromState(state, "_azureStorageAccounts", {inProgress: true}),
        _timeline: getPropFromState(state, "_timeline", {inProgress: true}),
        _dataBrokers: getPropFromState(state, "_dataBrokers", {inProgress: true}),
        _licenseSettings: getPropFromState(state, "_licenseSettings", {inProgress: true}),
        _licenses: getPropFromState(state, "_licenses", {inProgress: true}),
        _createDataBroker: getPropFromState(state, "_createDataBroker", {inProgress: true}),
        _dialog: state.syncReducer.dialog,
        _config: getPropFromState(state, '_config', {}),
        _popover: state.syncReducer.popover,
        _messages: getPropFromState(state, "_messages", {}),
        _showExports: getPropFromState(state, "_showExports", {}),
        _showShares: getPropFromState(state, "_showShares", {}),
        _showBuckets: getPropFromState(state, "_showBuckets", {}),
        _showAzureBlobContainers: getPropFromState(state, "_showAzureBlobContainers", {}),
        _showSGBuckets: getPropFromState(state, "_showSGBuckets", {}),
        _showGCPBuckets: getPropFromState(state, "_showGCPBuckets", {}),
        _relationships: getPropFromState(state, "_relationships", {inProgress: true, firstTime: true}),
        _wizard: getPropFromState(state, "_wizard", null),
        _ui: state.syncReducer._ui,
        _generalErrors: buildErrorsList(state),
        _costTimes: getPropFromState(state, "_costTimes", {inProgress: true}),
        _cost: getPropFromState(state, "_cost", {inProgress: true}),
        ownProps: ownProps,
        forms: state.form,
        routing: state.router,
        _routing: getPropFromState(state, "_routing", {}),
    }, ownProps)
}