import _ from "lodash";
import * as actions from './../action_creators_consts.js';
import {PAYLOAD_UPDATE_RELATIONSHIP} from "./../consts";
import numeral from "numeral";
import {matchTypeToSourceStep} from "./../utils/sync-utils"
import {RESPONSE_STATUS_DONE, RESPONSE_STATUS_RUNNING} from "../consts";
var FifoArray = require('fifo-array');


export const formatBytes = (bytes) =>{
    if(!bytes) return {size: "0", suffix: "B"};
    let formattedCapacity = numeral(bytes).format('0.[00] ib');
    formattedCapacity = formattedCapacity !== 0 ?  formattedCapacity.replace("i", "") : formattedCapacity;
    const returned = {
        size :  (formattedCapacity.split(" ")[0]) ,
        suffix : formattedCapacity.split(" ")[1]
    };
    return returned;
};

export const formatCount = (count) => {
    if(!count) return {size: "0", suffix: ""};
    let formattedCount = numeral(count).format('0.[0] a');
    const returned = {
        size :  (formattedCount.split(" ")[0]) ,
        suffix : formattedCount.split(" ")[1]
    };
    return returned;
};

const isItSourceStepByType = (type, step) => {
    if(!_.has(matchTypeToSourceStep, type)) {alert(type+ " type is not supported in isItSourceStepByType"); return false;}
    else return _.includes(matchTypeToSourceStep[type], step);
};


export const updateCurrentStep = (state) => {
    const identifyCurrentStep = (stepNames, steps) => {
        return stepNames.find((stepName, index) => {
            return index+2 > stepNames.length || !steps[stepNames[index+1]]
        })
    };
    const identifiedStep = identifyCurrentStep(state._wizard.stepNames, state._wizard.steps);
    //console.log(`@@@@@@@@@@@@@@@@@@@@@@@\n stepNames: ${state._wizard.stepNames}\nsteps:${JSON.stringify(state._wizard.steps)}\nidentifiedStep: ${identifiedStep}\n@@@@@@@@@@@@@@@@@@@@@@@@`);
    return state.update("_wizard", _wizard => _wizard.set("currentStep", identifiedStep).set("isSourceStep", isItSourceStepByType(_wizard.wizardType, identifiedStep)));
};

export const updateErrorsPanelStatus = (state) => {
    if (_.isEmpty(state._generalErrors._errorsMap)) return state.setIn(["_generalErrors","_errorsPanelOpen"], false).setIn(["_generalErrors","_errorsPanelSize"], {height: 0});
    return state.setIn(["_generalErrors","_errorsPanelOpen"], true);
};


export const buildStateFromAction = (action, postProcessor = buildState => buildState) => {
    return postProcessor({
        id: action.id,
        failed: action.status === actions.STATUS_ERROR,
        inProgress: action.status === actions.STATUS_PROGRESS,
        succeeded: action.status === actions.STATUS_SUCCESS,
        data: action.status === actions.STATUS_SUCCESS ? action.payload : undefined,
        error: action.status === actions.STATUS_ERROR ? action.payload : undefined,
        //requestId: action.requestId
    });
};

export const updateLastRequestsForRelationshipsData = (state, action) => {
    const activityStatus = action.payload[PAYLOAD_UPDATE_RELATIONSHIP].activity.status;
    if(!state._relationships || !state._relationships.data) return state;
    const newS =  state.updateIn(["_relationships", "data"], relationships => {
        return relationships.map(relationship => {
            if(relationship.id === action.payload[PAYLOAD_UPDATE_RELATIONSHIP].id){
                relationship = _.cloneDeep(relationship);
                if(relationship && relationship.lastRequests){
                    const lastRequests = new FifoArray(3, relationship.lastRequests);
                    const curruntRequest = _.find(lastRequests , function(req) {
                        return req.id === action.payload.requestId;
                    });
                     if(curruntRequest){
                        curruntRequest.isDone = (activityStatus === RESPONSE_STATUS_DONE);
                    }
                    //new requestId
                    else{
                        lastRequests.push({
                            id : action.payload.requestId,
                            isDone : (activityStatus === RESPONSE_STATUS_DONE)
                        });
                    }
                    relationship.lastRequests = lastRequests;
                }
            }
            return relationship;
        })
    });
    return newS;
};

export const setActivityInProgress = (activity) => {

    const newAct = {
        bytesCopied: 0,
        bytesFailed: 0,
        bytesMarkedForGrace: 0,
        bytesMarkedForRemove: 0,
        bytesRemoved: 0,
        bytesRemovedFailed: 0,
        dirsCopied: 0,
        dirsFailed: 0,
        dirsMarkedForCopy: 0,
        dirsMarkedForGrace: 0,
        dirsMarkedForRemove: 0,
        dirsRemoved: 0,
        filesCopied: 0,
        filesFailed: 0,
        filesMarkedForCopy: 0,
        filesMarkedForGrace: 0,
        filesMarkedforRemove: 0,
        filesRemoved: 0,
        filesRemovedFailed: 0,
        filesMarkedForIgnore: 0,
        progress: 0,
        status: RESPONSE_STATUS_RUNNING,
        type: "Sync"
    };
    return Object.assign({},activity, newAct);

};

export const updateKeyWithBuiltState = (state, action, key, postProcessor) => {
    const originalState = state[key];
    if(originalState && action.status !== actions.STATUS_PROGRESS && originalState.id !== action.id){
        console.log(`%c handleAction discarding action @ ${action.type} since the id doesn't match the id of the same action in the store`, "font-weight: bold;");
        return state;
    }

    return state.set(key, buildStateFromAction(action, postProcessor));
};

export const addUnhandledMessages = ((state, action) => {
    action = _.cloneDeep(action);
    action.unhandledCounter = action.unhandledCounter ? action.unhandledCounter + 1 : 1;
    if (action.unhandledCounter >= 5) {
        console.log(`%c addUnhandledMessages action @ ${action.type} wasn't handled for at least 5 messages, discarding`, "font-weight: bold;");
        return state;
    }
    else return state.update("unhandledMessages", unhandledMessages => unhandledMessages.concat(action));
});




